export declare enum State {
    New = 0,
    Learning = 1,
    Review = 2,
    Relearning = 3
}
export declare enum Rating {
    Again = 1,
    Hard = 2,
    Good = 3,
    Easy = 4
}
export declare class ReviewLog {
    rating: Rating;
    scheduled_days: number;
    elapsed_days: number;
    review: Date;
    state: State;
    constructor(rating: Rating, scheduled_days: number, elapsed_days: number, review: Date, state: State);
}
export declare class Card {
    due: Date;
    stability: number;
    difficulty: number;
    elapsed_days: number;
    scheduled_days: number;
    reps: number;
    lapses: number;
    state: State;
    last_review: Date;
    constructor();
}
export declare class SchedulingInfo {
    card: Card;
    review_log: ReviewLog;
    constructor(card: Card, review_log: ReviewLog);
}
declare class SchedulingCards {
    again: Card;
    hard: Card;
    good: Card;
    easy: Card;
    constructor(card: Card);
    update_state(state: State): void;
    schedule(now: Date, hard_interval: number, good_interval: number, easy_interval: number): void;
    record_log(card: Card, now: Date): Record<Rating, SchedulingInfo>;
}
declare class Params {
    request_retention: number;
    maximum_interval: number;
    w: Array<number>;
    constructor();
}
export declare class FSRS {
    p: Params;
    constructor();
    repeat(card: Card, now: Date): Record<number, SchedulingInfo>;
    init_ds(s: SchedulingCards): void;
    next_ds(s: SchedulingCards, last_d: number, last_s: number, retrievability: number): void;
    init_stability(r: number): number;
    init_difficulty(r: number): number;
    next_interval(s: number): number;
    next_difficulty(d: number, r: number): number;
    mean_reversion(init: number, current: number): number;
    next_recall_stability(d: number, s: number, r: number, rating: number): number;
    next_forget_stability(d: number, s: number, r: number): number;
}
export {};
